package ru.skipy.tests.ui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * OpaqueTest - <code>opaque</code> property demo
 *
 * @author Eugene Matyushkin aka Skipy
 * @version $Id: $
 * @since 12.11.2010
 */
public class OpaqueTest extends JFrame {

    private static final DateFormat DF = new SimpleDateFormat("HH:mm:ss.SSS");

    OpaqueTest() {
        super("Opaque property test");
        ContentPane cp = new ContentPane();
        final JButton btn = new JButton("Close") {
            /**
             * Paints component and logs painting events
             * @param g graphic context
             */
            @Override
            protected void paintComponent(Graphics g) {
                logRepaint("JButton", getLocation(), getSize(), g.getClip());
                super.paintComponent(g);
            }
        };
        btn.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                System.exit(0);
            }
        });
        cp.add(btn, BorderLayout.CENTER);
        final JCheckBox check = new JCheckBox("Opaque components", true) {
            /**
             * Paints component and logs painting events
             * @param g graphic context
             */
            @Override
            protected void paintComponent(Graphics g) {
                logRepaint("JCheckBox", getLocation(), getSize(), g.getClip());
                super.paintComponent(g);
            }
        };
        check.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                btn.setOpaque(check.isSelected());
                check.setOpaque(check.isSelected());
            }
        });
        cp.add(check, BorderLayout.SOUTH);
        setContentPane(cp);
        setSize(300, 200);
        setLocationRelativeTo(null);
        setDefaultCloseOperation(EXIT_ON_CLOSE);
    }

    /**
     * Logs painting event
     *
     * @param componentName component name
     * @param p             component position
     * @param size          component size
     * @param clip          painting clip
     */
    private static void logRepaint(String componentName, Point p, Dimension size, Shape clip) {
        System.out.println("Repainting " + componentName + " at " + DF.format(new Date()));
        System.out.println("  Component location: " + p);
        System.out.println("  Component size: " + size);
        System.out.println("  Clip rectangle: " + clip);
        System.out.println("");
    }

    public static void main(String[] args) {
        new OpaqueTest().setVisible(true);
    }

    /**
     * Content panel with painting events logging capability
     */
    private static class ContentPane extends JPanel {

        ContentPane() {
            super(new BorderLayout(10, 10));
            setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
            setBackground(new Color(192, 160, 255));
        }

        @Override
        protected void paintComponent(Graphics g) {
            logRepaint("ContentPane", getLocation(), getSize(), g.getClip());
            super.paintComponent(g);
        }
    }

}
