package ru.skipy.tests.ui.component.graph;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.Point2D;

/**
 * AbstractFunctionGraphPanel - parent for all graph panels
 *
 * @author Eugene Matyushkin aka Skipy
 * @version $Id: $
 * @since 11.10.2010
 */
public abstract class AbstractFunctionGraphPanel extends JPanel {

    /**
     * Gap between component content and border
     */
    protected static final int GAP = 5;

    /**
     * Graph area
     */
    protected double minX;
    protected double minY;
    protected double maxX;
    protected double maxY;

    /**
     * Graph function
     */
    protected Function function;

    /**
     * Function paint color
     */
    protected Color c;


    protected AbstractFunctionGraphPanel(GraphDescriptor descriptor, Color c) {
        setGraphDescriptorInt(descriptor);
        this.c = c;
        setBackground(Color.white);
    }

    /**
     * Calculates graph screen point
     *
     * @param x     function argument
     * @param stepX point-to-point step by x
     * @param stepY point-to-point step by y
     * @return calculated point
     */
    protected Point2D getGraphPoint(double x, double stepX, double stepY) {
        return new Point2D.Double((x - minX) / stepX, (function.getValue(x) - minY) / stepY);
    }

    /**
     * Sets graph descriptor
     *
     * @param descriptor descriptor to set
     */
    public void setGraphDescriptor(GraphDescriptor descriptor) {
        setGraphDescriptorInt(descriptor);
    }

    /**
     * Sets graph descriptor and initializes fields
     *
     * @param descriptor descriptor to set
     */
    private void setGraphDescriptorInt(GraphDescriptor descriptor) {
        this.function = descriptor;
        if (function != null) {
            this.minX = descriptor.getMinX();
            this.minY = descriptor.getMinY();
            this.maxX = descriptor.getMaxX();
            this.maxY = descriptor.getMaxY();
        } else {
            minX = minY = maxX = maxY = 0;
        }
    }
}
