package ru.skipy.tests.ui.component.graph;

import java.awt.*;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.geom.GeneralPath;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;

/**
 * FunctionGraphPanelGP - graph panel implementation using {@link java.awt.geom.GeneralPath}
 * and double buffering
 *
 * @author Eugene Matyushkin aka Skipy
 * @version $Id: $
 * @since 08.10.2010
 */
public class FunctionGraphPanelGPDB extends AbstractFunctionGraphPanel {

    /**
     * Component back buffer
     */
    private BufferedImage buffer = null;

    public FunctionGraphPanelGPDB(GraphDescriptor descriptor, Color c) {
        super(descriptor, c);
        addComponentListener(new ComponentListenerImpl());
    }

    @Override
    protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        if (function == null) {
            return;
        }
        if (buffer == null) {
            rebuildBuffer();
        }
        g.drawImage(buffer, 0, 0, this);
    }

    /**
     * Removes old buffer to regenerate it after function change
     * even if size is not changed
     *
     * @param descriptor descriptor to set
     */
    @Override
    public void setGraphDescriptor(GraphDescriptor descriptor) {
        super.setGraphDescriptor(descriptor);
        buffer = null;
    }

    /**
     * Rebuilds buffer
     */
    private void rebuildBuffer() {
        if (function == null) {
            return;
        }
        int w = getWidth();
        int h = getHeight();
        buffer = new BufferedImage(w, h, BufferedImage.TYPE_INT_ARGB);
        if (w <= 2 * GAP || h <= 2 * GAP)
            return;
        Graphics2D g2d = buffer.createGraphics();
        int ew = w - 2 * GAP;
        int eh = h - 2 * GAP;
        double stepX = (maxX - minX) / ew;
        double stepY = (maxY - minY) / eh;
        g2d.setPaint(Color.darkGray);
        Line2D.Double xAxis = new Line2D.Double(GAP, GAP + maxY / stepY, GAP + ew, GAP + maxY / stepY);
        Line2D.Double yAxis = new Line2D.Double(GAP - minX / stepX, GAP, GAP - minX / stepX, GAP + eh);
        g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        g2d.draw(xAxis);
        g2d.draw(yAxis);
        GeneralPath graph = new GeneralPath();
        Point2D start = getGraphPoint(minX, stepX, stepY);
        graph.moveTo(GAP + start.getX(), GAP + eh - start.getY());
        for (int i = 1; i <= ew; i++) {
            Point2D p = getGraphPoint(i * stepX + minX, stepX, stepY);
            graph.lineTo(GAP + p.getX(), GAP + eh - p.getY());
        }
        g2d.setPaint(c);
        g2d.draw(graph);
    }

    /**
     * Component events listener to track size changes
     */
    private class ComponentListenerImpl extends ComponentAdapter {

        private Dimension lastSize = null;

        @Override
        public void componentShown(ComponentEvent e) {
            if (!getSize().equals(lastSize)) {
                rebuildBuffer();
                lastSize = getSize();
            }
        }

        @Override
        public void componentResized(ComponentEvent e) {
            if (!getSize().equals(lastSize)) {
                rebuildBuffer();
                lastSize = getSize();
            }
        }
    }
}
