package ru.skipy.tests.ui.mouse_paints;

import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

/**
 * SelectionPaintTest - Sample application to demonstrate selection creation techniques
 *
 * @author Eugene Matyushkin aka Skipy
 * @version $Id: $
 * @since 13.10.2010
 */
public class SelectionPaintTest extends JFrame {

    SelectionPaintTest() {
        super("Selection paint test");
        SelectablePanel sp = new SelectablePanel();
        JPanel cp = new JPanel(new BorderLayout());
        cp.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5),
                BorderFactory.createLineBorder(Color.red)));
        setContentPane(cp);
        cp.add(sp, BorderLayout.CENTER);
        setSize(800, 700);
        setLocationRelativeTo(null);
        setDefaultCloseOperation(EXIT_ON_CLOSE);
    }

    public static void main(String[] args) {
        new SelectionPaintTest().setVisible(true);
    }

    /**
     * Panel with the possiblity of selection creation
     */
    static class SelectablePanel extends JPanel {

        /**
         * Selection component
         */
        private SelectionComponent sc = new SelectionComponent(Color.black, 128);

        /**
         * Sample background image
         */
        private Image bg = null;

        SelectablePanel() {
            bg = Toolkit.getDefaultToolkit().createImage(getClass().getResource("/rb07.jpg"));
            MouseHandler mh = new MouseHandler();
            addMouseListener(mh);
            addMouseMotionListener(mh);
            setBackground(Color.white);
            setLayout(null);
            add(sc);
        }

        /**
         * Paints backgrouns image
         *
         * @param g graphic context
         */
        @Override
        protected void paintComponent(Graphics g) {
            super.paintComponent(g);
            if (bg != null)
                while (!g.drawImage(bg, (getWidth() - bg.getWidth(this)) / 2,
                        (getHeight() - bg.getHeight(this)) / 2, this)) {
                }
        }

        /**
         * Mouse events handler to change selection
         */
        class MouseHandler extends MouseAdapter {
            @Override
            public void mousePressed(MouseEvent e) {
                sc.setStart(e.getX(), e.getY());
            }

            @Override
            public void mouseReleased(MouseEvent e) {
                sc.selected();
            }

            @Override
            public void mouseDragged(MouseEvent e) {
                sc.setEnd(e.getX(), e.getY());
            }
        }

        /**
         * Selection component
         */
        class SelectionComponent extends JPanel {

            private Color selectionColor;
            private Color borderColor;

            private int startX = 0;
            private int startY = 0;

            SelectionComponent(Color color, int alpha) {
                setOpaque(false);
                this.selectionColor = new Color(color.getRed(),
                        color.getGreen(),
                        color.getBlue(),
                        alpha);
                this.borderColor = color;
            }

            /**
             * Paints component - fills it with semi-transparent color
             *
             * @param g graphic context
             */
            @Override
            protected void paintComponent(Graphics g) {
                super.paintComponent(g);
                g.setColor(selectionColor);
                g.fillRect(0, 0, getWidth(), getHeight());
            }

            private void setStart(int x, int y) {
                startX = x;
                startY = y;
                setBounds(x, y, 0, 0);
                setBorder(null);
            }

            private void setEnd(int x, int y) {
                int tlX = Math.min(x, startX);
                int tlY = Math.min(y, startY);
                int brX = Math.max(x, startX);
                int brY = Math.max(y, startY);
                setBounds(tlX, tlY, brX - tlX, brY - tlY);
            }

            private void selected() {
                setBorder(BorderFactory.createLineBorder(borderColor));
            }
        }
    }
}
